//-----------------------------------------------------------------------------
//
//                      XMM Science Analysis System (SAS)
//                     (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	  xcolorcod.cc	--- color coding of spectral images
//
//	DESCRIPTION:
//		Construct RGB image data cube.
//
//	CHANGE HISTORY:
//		MB, 1999-??-??: 1.0.0: - first version
//		UL, 2000-07-17: 2.0.0: - general overhaul in the process of fixing
//								 SPR 855
//							   - is now a metatask that invokes evselect()
//								 to do the image creation
//		UL, 2001-04-04: 2.1.0: - added parameter "ppmfile"
//
//	AUTHORS:
//		Marco Beijersbergen (MB)
//		Uwe Lammers (UL)
// 
// $Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xcolorcod/src/xcolorcod.cc,v 1.6 2001/04/04 14:32:16 ulammers Exp $
//

#include <iostream>
#include <fstream>
#include "Dal.h"
#include "metatask.h"
#include "stringutil.h"
#include "Color.h"
#include "TableBasedPseudoRGBImage.h"

#include "xcolorcod.h"

class TableBasedColorCurve
	: public LinearlyInterpolatedPointsCurve<double, Color>
{
public:
	TableBasedColorCurve(const string &colorTable,
						 const string &colorECol,
						 const string &redCol,
						 const string &greenCol,
						 const string &blueCol) :
		LinearlyInterpolatedPointsCurve<double, Color>()
	{
		Table* colTab = dataSetServer->open(colorTable, Dal::Read)->
							table(colorTable);
		ColumnData	*edata = colTab->column(colorECol)->data(),
					*rdata = colTab->column(redCol)->data(),
					*gdata = colTab->column(greenCol)->data(),
					*bdata = colTab->column(blueCol)->data();
		real32		*cole = edata->real32Data(),
					*red = rdata->real32Data(),
					*green = gdata->real32Data(),
					*blue = bdata->real32Data();

		for (unsigned row=0; row<colTab->rows(); ++row)
			(*this)[cole[row]] = Color(red[row], green[row], blue[row]);

		delete edata;
		delete rdata;
		delete gdata;
		delete bdata;
		dataSetServer->close(colTab->parent());
	}
};


void xcolorcod()
{
	ParameterInfo *p = Info::Task::current()->parameters();

	//	construct color curve
	TableBasedColorCurve colorCurve(
								p->parameter("colortable")->stringValue(),
								p->parameter("colorecolumn")->stringValue(),
								p->parameter("redcolumn")->stringValue(),
								p->parameter("greencolumn")->stringValue(),	
								p->parameter("bluecolumn")->stringValue());

	//	construct temporary data set with R/G/B columns
	TableBasedPseudoRGBImage rgbPseudoImage(
								p->parameter("table")->stringValue(),
								p->parameter("xcolumn")->stringValue(),
								p->parameter("ycolumn")->stringValue(),
								p->parameter("ecolumn")->stringValue(),
								colorCurve);

	//	construct real color image
	ColorImage *rgbImage = rgbPseudoImage.toColorImage();

	//	log-scaling/normalizing
	if (uppercase(p->parameter("scale")->stringValue()) == "LOG")
		rgbImage->log(p->parameter("decades")->realValue());
	rgbImage->normalize(255);
	if (p->parameter("withcolorset")->boolValue())
		rgbImage->writeDataSet(p->parameter("colorset")->stringValue(),
							   p->parameter("ascube")->boolValue());
	else {
		string ppmName = p->parameter("ppmfile")->stringValue();
		ostream *os = &cout;
		ofstream ofs;
		if (uppercase(ppmName) != "STDOUT") {
			ofs.open(ppmName.c_str());
			os = &ofs;
		}
		rgbImage->writePPM(*os);
	}
} // xcolorcod()
